@@ -8,8 +8,8 @@ from collections import defaultdict
 
 import numpy as np
 import pandas as pd
-from rdt import HyperTransformer, transformers
 
+from sdv.constraints import Constraint
 from sdv.metadata import visualization
 from sdv.metadata.errors import MetadataError
 
@@ -21,9 +21,7 @@ def _read_csv_dtypes(table_meta):
     dtypes = dict()
     for name, field in table_meta['fields'].items():
         field_type = field['type']
-        if field_type == 'categorical':
-            dtypes[name] = str
-        elif field_type == 'id' and field.get('subtype', 'integer') == 'string':
+        if field_type == 'id' and field.get('subtype', 'integer') == 'string':
             dtypes[name] = str
 
     return dtypes
@@ -73,7 +71,6 @@ class Metadata:
     """
 
     _child_map = None
-    _hyper_transformers = None
     _metadata = None
     _parent_map = None
 
@@ -182,7 +179,6 @@ class Metadata:
         else:
             self._metadata = {'tables': {}}
 
-        self._hyper_transformers = dict()
         self._analyze_relationships()
 
     def get_children(self, table_name):
@@ -299,10 +295,10 @@ class Metadata:
         """
         return self.get_table_meta(table_name).get('primary_key')
 
-    def get_foreign_key(self, parent, child):
-        """Get the name of the field in the child that is a foreign key to parent.
+    def get_foreign_keys(self, parent, child):
+        """Get the name of all the fields in the child that are foreign keys to this parent.
 
-        If there is no relationship between the two tables, a ``ValueError`` is raised.
+        If there is no relationship between the two tables an empty list is returned.
 
         Args:
             parent (str):
@@ -311,19 +307,16 @@ class Metadata:
                 Name of the child table.
 
         Returns:
-            str or None:
-                Foreign key field name.
-
-        Raises:
-            ValueError:
-                If the relationship does not exist.
+            list[str]:
+                List of foreign key names.
         """
+        foreign_keys = []
         for name, field in self.get_fields(child).items():
             ref = field.get('ref')
             if ref and ref['table'] == parent:
-                return name
+                foreign_keys.append(name)
 
-        raise ValueError('{} is not parent of {}'.format(parent, child))
+        return foreign_keys
 
     def load_table(self, table_name):
         """Load the data of the indicated table as a DataFrame.
@@ -363,7 +356,7 @@ class Metadata:
             for table_name in tables or self.get_tables()
         }
 
-    def get_dtypes(self, table_name, ids=False):
+    def get_dtypes(self, table_name, ids=False, errors=None):
         """Get a ``dict`` with the ``dtypes`` for each field of a given table.
 
         Args:
@@ -381,6 +374,7 @@ class Metadata:
                 If a field has an invalid type or subtype or if the table does not
                 exist in this metadata.
         """
+        errors = [] if errors is None else errors
         dtypes = dict()
         table_meta = self.get_table_meta(table_name)
         for name, field in table_meta['fields'].items():
@@ -388,150 +382,27 @@ class Metadata:
             field_subtype = field.get('subtype')
             dtype = self._DTYPES.get((field_type, field_subtype))
             if not dtype:
-                raise MetadataError(
+                errors.append(
                     'Invalid type and subtype combination for field {}: ({}, {})'.format(
                         name, field_type, field_subtype)
                 )
+            else:
+                if ids and field_type == 'id':
+                    if (name != table_meta.get('primary_key')) and not field.get('ref'):
+                        for child_table in self.get_children(table_name):
+                            if name in self.get_foreign_keys(table_name, child_table):
+                                break
 
-            if ids and field_type == 'id':
-                if (name != table_meta.get('primary_key')) and not field.get('ref'):
-                    for child_table in self.get_children(table_name):
-                        if name == self.get_foreign_key(table_name, child_table):
-                            break
-
-            if ids or (field_type != 'id'):
-                dtypes[name] = dtype
+                if ids or (field_type != 'id'):
+                    dtypes[name] = dtype
 
         return dtypes
 
-    def _get_pii_fields(self, table_name):
-        """Get the ``pii_category`` for each field of the table that contains PII.
-
-        Args:
-            table_name (str):
-                Table name for which to get the pii fields.
-
-        Returns:
-            dict:
-                pii field names and categories.
-        """
-        pii_fields = dict()
-        for name, field in self.get_table_meta(table_name)['fields'].items():
-            if field['type'] == 'categorical' and field.get('pii', False):
-                pii_fields[name] = field['pii_category']
-
-        return pii_fields
-
-    @staticmethod
-    def _get_transformers(dtypes, pii_fields):
-        """Create the transformer instances needed to process the given dtypes.
-
-        Temporary drop-in replacement of ``HyperTransformer._analyze`` method,
-        before RDT catches up.
-
-        Args:
-            dtypes (dict):
-                mapping of field names and dtypes.
-            pii_fields (dict):
-                mapping of pii field names and categories.
-
-        Returns:
-            dict:
-                mapping of field names and transformer instances.
-        """
-        transformers_dict = dict()
-        for name, dtype in dtypes.items():
-            dtype = np.dtype(dtype)
-            if dtype.kind == 'i':
-                transformer = transformers.NumericalTransformer(dtype=int)
-            elif dtype.kind == 'f':
-                transformer = transformers.NumericalTransformer(dtype=float)
-            elif dtype.kind == 'O':
-                anonymize = pii_fields.get(name)
-                transformer = transformers.CategoricalTransformer(anonymize=anonymize)
-            elif dtype.kind == 'b':
-                transformer = transformers.BooleanTransformer()
-            elif dtype.kind == 'M':
-                transformer = transformers.DatetimeTransformer()
-            else:
-                raise ValueError('Unsupported dtype: {}'.format(dtype))
-
-            LOGGER.info('Loading transformer %s for field %s',
-                        transformer.__class__.__name__, name)
-            transformers_dict[name] = transformer
-
-        return transformers_dict
-
-    def _load_hyper_transformer(self, table_name):
-        """Create and return a new ``rdt.HyperTransformer`` instance for a table.
-
-        First get the ``dtypes`` and ``pii fields`` from a given table, then use
-        those to build a transformer dictionary to be used by the ``HyperTransformer``.
-
-        Args:
-            table_name (str):
-                Table name for which to load the HyperTransformer.
-
-        Returns:
-            rdt.HyperTransformer:
-                Instance of ``rdt.HyperTransformer`` for the given table.
-        """
-        dtypes = self.get_dtypes(table_name)
-        pii_fields = self._get_pii_fields(table_name)
-        transformers_dict = self._get_transformers(dtypes, pii_fields)
-        return HyperTransformer(transformers=transformers_dict)
-
-    def transform(self, table_name, data):
-        """Transform data for a given table.
-
-        If the ``HyperTransformer`` for a table is ``None`` it is created.
-
-        Args:
-            table_name (str):
-                Name of the table that is being transformer.
-            data (pandas.DataFrame):
-                Table data.
-
-        Returns:
-            pandas.DataFrame:
-                Transformed data.
-        """
-        hyper_transformer = self._hyper_transformers.get(table_name)
-        if hyper_transformer is None:
-            hyper_transformer = self._load_hyper_transformer(table_name)
-            fields = list(hyper_transformer.transformers.keys())
-            hyper_transformer.fit(data[fields])
-            self._hyper_transformers[table_name] = hyper_transformer
-
-        hyper_transformer = self._hyper_transformers.get(table_name)
-        fields = list(hyper_transformer.transformers.keys())
-        return hyper_transformer.transform(data[fields])
-
-    def reverse_transform(self, table_name, data):
-        """Reverse the transformed data for a given table.
-
-        Args:
-            table_name (str):
-                Name of the table to reverse transform.
-            data (pandas.DataFrame):
-                Data to be reversed.
-
-        Returns:
-            pandas.DataFrame
-        """
-        hyper_transformer = self._hyper_transformers[table_name]
-        reversed_data = hyper_transformer.reverse_transform(data)
-
-        for name, dtype in self.get_dtypes(table_name, ids=True).items():
-            reversed_data[name] = reversed_data[name].dropna().astype(dtype)
-
-        return reversed_data
-
     # ################### #
     # Metadata Validation #
     # ################### #
 
-    def _validate_table(self, table_name, table_meta, table_data=None):
+    def _validate_table(self, table_name, table_meta, table_data=None, errors=None):
         """Validate table metadata.
 
         Validate the type and subtype combination for each field in ``table_meta``.
@@ -558,7 +429,8 @@ class Metadata:
                 If there is any error in the metadata or the data does not
                 match the metadata description.
         """
-        dtypes = self.get_dtypes(table_name, ids=True)
+        errors = [] if errors is None else errors
+        dtypes = self.get_dtypes(table_name, ids=True, errors=errors)
 
         # Primary key field exists and its type is 'id'
         primary_key = table_meta.get('primary_key')
@@ -566,10 +438,11 @@ class Metadata:
             pk_field = table_meta['fields'].get(primary_key)
 
             if not pk_field:
-                raise MetadataError('Primary key is not an existing field.')
-
-            if pk_field['type'] != 'id':
-                raise MetadataError('Primary key is not of type `id`.')
+                errors.append(
+                    f'Invalid primary key: "{primary_key}" not found in table "{table_name}"')
+            elif pk_field['type'] != 'id':
+                errors.append(
+                    f'Primary key "{primary_key}" of table "{table_name}" not of type "id"')
 
         if table_data is not None:
             for column in table_data:
@@ -578,28 +451,36 @@ class Metadata:
                     table_data[column].dropna().astype(dtype)
                 except KeyError:
                     message = 'Unexpected column in table `{}`: `{}`'.format(table_name, column)
-                    raise MetadataError(message) from None
+                    errors.append(message)
                 except ValueError as ve:
                     message = 'Invalid values found in column `{}` of table `{}`: `{}`'.format(
                         column, table_name, ve)
-                    raise MetadataError(message) from None
+                    errors.append(message)
 
             # assert all dtypes are in data
             if dtypes:
-                raise MetadataError(
+                errors.append(
                     'Missing columns on table {}: {}.'.format(table_name, list(dtypes.keys()))
                 )
 
-    def _validate_circular_relationships(self, parent, children=None):
+    def _validate_circular_relationships(self, parent, children=None, errors=None, parents=None):
         """Validate that there is no circular relatioship in the metadata."""
+        errors = [] if errors is None else errors
+        parents = set() if parents is None else parents
         if children is None:
             children = self.get_children(parent)
 
         if parent in children:
-            raise MetadataError('Circular relationship found for table "{}"'.format(parent))
+            error = 'Circular relationship found for table "{}"'.format(parent)
+            errors.append(error)
 
         for child in children:
-            self._validate_circular_relationships(parent, self.get_children(child))
+            if child in parents:
+                break
+
+            parents.add(child)
+            self._validate_circular_relationships(
+                parent, self.get_children(child), errors, parents)
 
     def validate(self, tables=None):
         """Validate this metadata.
@@ -633,17 +514,21 @@ class Metadata:
         if tables and not isinstance(tables, dict):
             tables = self.load_tables()
 
+        errors = []
         for table_name, table_meta in tables_meta.items():
             if tables:
                 table = tables.get(table_name)
                 if table is None:
-                    raise MetadataError('Table `{}` not found in tables'.format(table_name))
+                    errors.append('Table `{}` not found in tables'.format(table_name))
 
             else:
                 table = None
 
-            self._validate_table(table_name, table_meta, table)
-            self._validate_circular_relationships(table_name)
+            self._validate_table(table_name, table_meta, table, errors)
+            self._validate_circular_relationships(table_name, errors=errors)
+
+        if errors:
+            raise MetadataError('Invalid Metadata specification:\n - ' + '\n - '.join(errors))
 
     def _check_field(self, table, field, exists=False):
         """Validate the existance of the table and existance (or not) of field."""
@@ -743,7 +628,7 @@ class Metadata:
         }
         table_meta['primary_key'] = field
 
-    def add_relationship(self, parent, child, foreign_key=None):
+    def add_relationship(self, parent, child, foreign_key=None, validate=True):
         """Add a new relationship between the parent and child tables.
 
         The relationship is created by adding a reference (``ref``) on the ``foreign_key``
@@ -757,6 +642,9 @@ class Metadata:
             foreign_key (str):
                 Field in the child table through which the relationship is created.
                 If ``None``, use the parent primary key name.
+            validate (bool):
+                Whether to validate metadata after adding this relationship or not.
+                Defaults to ``True``.
 
         Raises:
             ValueError:
@@ -768,6 +656,9 @@ class Metadata:
                     * The child table already has a parent.
                     * The new relationship closes a relationship circle.
         """
+        # Make a backup
+        metadata_backup = copy.deepcopy(self._metadata)
+
         # Validate tables exists
         self.get_table_meta(parent)
         self.get_table_meta(child)
@@ -789,10 +680,6 @@ class Metadata:
             raise ValueError(
                 'Field "{}.{}" already defines a relationship'.format(child, foreign_key))
 
-        grandchildren = self.get_children(child)
-        if grandchildren:
-            self._validate_circular_relationships(parent, grandchildren)
-
         # Make sure that the parent key is an id
         if parent_key_meta['type'] != 'id':
             parent_key_meta['subtype'] = self._get_key_subtype(parent_key_meta)
@@ -810,20 +697,18 @@ class Metadata:
         if child_key_meta['subtype'] != parent_key_meta['subtype']:
             raise ValueError('Parent and Child key subtypes mismatch')
 
-        # Make a backup
-        metadata_backup = copy.deepcopy(self._metadata)
-
         self._metadata['tables'][parent]['fields'][primary_key] = parent_key_meta
         self._metadata['tables'][child]['fields'][foreign_key] = child_key_meta
 
         # Re-analyze the relationships
         self._analyze_relationships()
 
-        try:
-            self.validate()
-        except MetadataError:
-            self._metadata = metadata_backup
-            raise
+        if validate:
+            try:
+                self.validate()
+            except MetadataError:
+                self._metadata = metadata_backup
+                raise
 
     def _get_field_details(self, data, fields):
         """Get or build all the fields metadata.
@@ -861,7 +746,7 @@ class Metadata:
         return fields_metadata
 
     def add_table(self, name, data=None, fields=None, fields_metadata=None,
-                  primary_key=None, parent=None, foreign_key=None):
+                  primary_key=None, parent=None, foreign_key=None, constraints=None):
         """Add a new table to this metadata.
 
         ``fields`` list can be a mixture of field names, which will be build automatically
@@ -892,7 +777,10 @@ class Metadata:
             parent (str):
                 Table name to refere a foreign key field. Defaults to ``None``.
             foreign_key (str):
-                Foreing key field name to ``parent`` table primary key. Defaults to ``None``.
+                Foreign key field name to ``parent`` table primary key. Defaults to ``None``.
+            constraints (list[Constraint, dict]):
+                List of Constraint objects or dicts representing the constraints for the
+                given table.
 
         Raises:
             ValueError:
@@ -928,6 +816,16 @@ class Metadata:
 
         self._metadata['tables'][name] = table_metadata
 
+        if constraints:
+            meta_constraints = []
+            for constraint in constraints:
+                if isinstance(constraint, Constraint):
+                    meta_constraints.append(constraint.to_dict())
+                else:
+                    meta_constraints.append(constraint)
+
+            table_metadata['constraints'] = meta_constraints
+
         try:
             if primary_key:
                 self.set_primary_key(name, primary_key)
@@ -967,11 +865,12 @@ class Metadata:
         tables = self.get_tables()
         relationships = [
             '    {}.{} -> {}.{}'.format(
-                table, self.get_foreign_key(parent, table),
+                table, foreign_key,
                 parent, self.get_primary_key(parent)
             )
             for table in tables
             for parent in list(self.get_parents(table))
+            for foreign_key in self.get_foreign_keys(parent, table)
         ]
 
         return (
@@ -986,7 +885,7 @@ class Metadata:
             '\n'.join(relationships)
         )
 
-    def visualize(self, path=None):
+    def visualize(self, path=None, names=True, details=True):
         """Plot metadata usign graphviz.
 
         Generate a plot using graphviz.
@@ -998,5 +897,7 @@ class Metadata:
                 supported extension. If ``None`` do not save the plot and
                 just return the ``graphviz.Digraph`` object.
                 Defaults to ``None``.
+            names (bool):
+                Whether to add names to the diagram or not. Defaults to ``True``
         """
-        return visualization.visualize(self, path)
+        return visualization.visualize(self, path, names=names, details=details)
