@@ -3,14 +3,16 @@
 import copy
 import json
 import logging
+import warnings
 
 import numpy as np
 import pandas as pd
 import rdt
 from faker import Faker
 
-from sdv.constraints.base import Constraint
-from sdv.constraints.errors import MissingConstraintColumnError
+from sdv.constraints import Constraint
+from sdv.constraints.errors import (
+    FunctionError, MissingConstraintColumnError, MultipleConstraintsErrors)
 from sdv.metadata.errors import MetadataError, MetadataNotFittedError
 from sdv.metadata.utils import strings_from_regex
 
@@ -42,14 +44,13 @@ class Table:
             Dictinary specifying which transformers to use for each field.
             Available transformers are:
 
-                * ``integer``: Uses a ``NumericalTransformer`` of dtype ``int``.
-                * ``float``: Uses a ``NumericalTransformer`` of dtype ``float``.
-                * ``categorical``: Uses a ``CategoricalTransformer`` without gaussian noise.
-                * ``categorical_fuzzy``: Uses a ``CategoricalTransformer`` adding gaussian noise.
-                * ``one_hot_encoding``: Uses a ``OneHotEncodingTransformer``.
-                * ``label_encoding``: Uses a ``LabelEncodingTransformer``.
-                * ``boolean``: Uses a ``BooleanTransformer``.
-                * ``datetime``: Uses a ``DatetimeTransformer``.
+                * ``FloatFormatter``: Uses a ``FloatFormatter`` for numerical data.
+                * ``FrequencyEncoder``: Uses a ``FrequencyEncoder`` without gaussian noise.
+                * ``FrequencyEncoder_noised``: Uses a ``FrequencyEncoder`` adding gaussian noise.
+                * ``OneHotEncoder``: Uses a ``OneHotEncoder``.
+                * ``LabelEncoder``: Uses a ``LabelEncoder``.
+                * ``BinaryEncoder``: Uses a ``BinaryEncoder``.
+                * ``UnixTimestampEncoder``: Uses a ``UnixTimestampEncoder``.
 
         anonymize_fields (dict[str, str]):
             Dict specifying which fields to anonymize and what faker
@@ -84,31 +85,47 @@ class Table:
             The columns in the dataframe which are constant within each
             group/entity. These columns will be provided at sampling time
             (i.e. the samples will be conditioned on the context variables).
+        learn_rounding_scheme (bool):
+            Define rounding scheme for ``FloatFormatter``. If ``True``, the data returned by
+            ``reverse_transform`` will be rounded to that place. Defaults to ``True``.
+        enforce_min_max_values (bool):
+            Specify whether or not to clip the data returned by ``reverse_transform`` of
+            the numerical transformer, ``FloatFormatter``, to the min and max values seen
+            during ``fit``. Defaults to ``True``.
     """
 
     _hyper_transformer = None
-    _fakers = None
-    _constraint_instances = None
     _fields_metadata = None
     fitted = False
 
     _ANONYMIZATION_MAPPINGS = dict()
     _TRANSFORMER_TEMPLATES = {
-        'integer': rdt.transformers.NumericalTransformer(dtype=int),
-        'float': rdt.transformers.NumericalTransformer(dtype=float),
-        'categorical': rdt.transformers.CategoricalTransformer,
-        'categorical_fuzzy': rdt.transformers.CategoricalTransformer(fuzzy=True),
-        'one_hot_encoding': rdt.transformers.OneHotEncodingTransformer,
-        'label_encoding': rdt.transformers.LabelEncodingTransformer,
-        'boolean': rdt.transformers.BooleanTransformer,
-        'datetime': rdt.transformers.DatetimeTransformer(strip_constant=True),
+        'FloatFormatter': rdt.transformers.FloatFormatter(
+            learn_rounding_scheme=True,
+            enforce_min_max_values=True,
+            missing_value_replacement='mean',
+            model_missing_values=True,
+        ),
+        'FrequencyEncoder': rdt.transformers.FrequencyEncoder,
+        'FrequencyEncoder_noised': rdt.transformers.FrequencyEncoder(add_noise=True),
+        'OneHotEncoder': rdt.transformers.OneHotEncoder,
+        'LabelEncoder': rdt.transformers.LabelEncoder,
+        'LabelEncoder_noised': rdt.transformers.LabelEncoder(add_noise=True),
+        'BinaryEncoder': rdt.transformers.BinaryEncoder(
+            missing_value_replacement=-1,
+            model_missing_values=True
+        ),
+        'UnixTimestampEncoder': rdt.transformers.UnixTimestampEncoder(
+            missing_value_replacement='mean',
+            model_missing_values=True,
+        )
     }
     _DTYPE_TRANSFORMERS = {
-        'i': 'integer',
-        'f': 'float',
-        'O': 'one_hot_encoding',
-        'b': 'boolean',
-        'M': 'datetime',
+        'i': 'FloatFormatter',
+        'f': 'FloatFormatter',
+        'O': 'OneHotEncoder',
+        'b': 'BinaryEncoder',
+        'M': 'UnixTimestampEncoder',
     }
     _DTYPES_TO_TYPES = {
         'i': {
@@ -141,10 +158,28 @@ class Table:
         ('id', 'string'): 'str'
     }
 
-    def _get_faker(self, category):
-        """Return the faker object to anonymize data.
+    @staticmethod
+    def _get_faker(field_metadata):
+        """Return the faker object with localisaton set if specified in field_metadata.
 
         Args:
+            field_metadata (dict):
+                Metadata for field to read localisation from if set in `pii_locales`.
+
+        Returns:
+            Faker object:
+                The Faker object to anonymize the data in the field using its functions.
+        """
+        pii_locales = field_metadata.get('pii_locales', None)
+        return Faker(locale=pii_locales)
+
+    @staticmethod
+    def _get_faker_method(faker, category):
+        """Return the faker function to anonymize data.
+
+        Args:
+            faker (Faker object):
+                The faker object created to get functions from.
             category (str or tuple):
                 Fake category to use. If a tuple is passed, the first element is
                 the category and the rest are additional arguments for the Faker.
@@ -163,23 +198,68 @@ class Table:
             args = tuple()
 
         try:
-            faker_method = getattr(Faker(), category)
+            if args:
+                def _faker():
+                    return getattr(faker, category)(*args)
 
-            if not args:
-                return faker_method
-
-            def faker():
-                return faker_method(*args)
-
-            return faker
+            else:
+                def _faker():
+                    return getattr(faker, category)()
 
+            return _faker
         except AttributeError:
             raise ValueError('Category "{}" couldn\'t be found on faker'.format(category))
 
+    @staticmethod
+    def _get_fake_values(field_metadata, num_values):
+        """Return the anonymized values from Faker.
+
+        Args:
+            field_metadata (dict):
+                Metadata for field to read localisation from if set in `pii_locales`.
+                And to read the faker category from `pii_category`.
+            num_values (int):
+                Number of values to create.
+
+        Returns:
+            generator:
+                Generator containing the anonymized values.
+        """
+        faker = Table._get_faker(field_metadata)
+        faker_method = Table._get_faker_method(faker, field_metadata['pii_category'])
+        return (
+            faker_method()
+            for _ in range(num_values)
+        )
+
+    def _update_transformer_templates(self, learn_rounding_scheme, enforce_min_max_values):
+        custom_float_formatter = rdt.transformers.FloatFormatter(
+            missing_value_replacement='mean',
+            model_missing_values=True,
+            learn_rounding_scheme=learn_rounding_scheme,
+            enforce_min_max_values=enforce_min_max_values
+        )
+        self._transformer_templates.update({
+            'FloatFormatter': custom_float_formatter,
+        })
+
+    @staticmethod
+    def _load_constraints(constraints):
+        constraints = constraints or []
+        loaded_constraints = []
+        for constraint in constraints:
+            if isinstance(constraint, dict):
+                loaded_constraints.append(Constraint.from_dict(constraint))
+            else:
+                loaded_constraints.append(constraint)
+
+        return loaded_constraints
+
     def __init__(self, name=None, field_names=None, field_types=None, field_transformers=None,
                  anonymize_fields=None, primary_key=None, constraints=None,
                  dtype_transformers=None, model_kwargs=None, sequence_index=None,
-                 entity_columns=None, context_columns=None):
+                 entity_columns=None, context_columns=None,
+                 learn_rounding_scheme=True, enforce_min_max_values=True):
         self.name = name
         self._field_names = field_names
         self._field_types = field_types or {}
@@ -191,8 +271,12 @@ class Table:
         self._sequence_index = sequence_index
         self._entity_columns = entity_columns or []
         self._context_columns = context_columns or []
-        self._constraints = constraints or []
+        self._constraints = self._load_constraints(constraints)
+        self._constraints_to_reverse = []
         self._dtype_transformers = self._DTYPE_TRANSFORMERS.copy()
+        self._transformer_templates = self._TRANSFORMER_TEMPLATES.copy()
+        self._update_transformer_templates(learn_rounding_scheme, enforce_min_max_values)
+
         if dtype_transformers:
             self._dtype_transformers.update(dtype_transformers)
 
@@ -305,7 +389,7 @@ class Table:
 
         return fields_metadata
 
-    def _get_transformers(self, dtypes):
+    def _get_hypertransformer_config(self, dtypes):
         """Create the transformer instances needed to process the given dtypes.
 
         Args:
@@ -314,22 +398,25 @@ class Table:
 
         Returns:
             dict:
-                mapping of field names and transformer instances.
+                A dict containing the ``sdtypes`` and ``transformers`` config for the
+                ``rdt.HyperTransformer``.
         """
         transformers = dict()
+        sdtypes = dict()
         for name, dtype in dtypes.items():
+            dtype = np.dtype(dtype).kind
             field_metadata = self._fields_metadata.get(name, {})
-            transformer_template = field_metadata.get('transformer')
+            transformer_template = field_metadata.get(
+                'transformer', self._dtype_transformers[dtype])
+
             if transformer_template is None:
-                transformer_template = self._dtype_transformers[np.dtype(dtype).kind]
-                if transformer_template is None:
-                    # Skip this dtype
-                    continue
-
-                field_metadata['transformer'] = transformer_template
+                sdtypes[name] = self._DTYPES_TO_TYPES.get(dtype, {}).get('type', 'categorical')
+                transformers[name] = None
+                continue
 
+            field_metadata['transformer'] = transformer_template
             if isinstance(transformer_template, str):
-                transformer_template = self._TRANSFORMER_TEMPLATES[transformer_template]
+                transformer_template = self._transformer_templates[transformer_template]
 
             if isinstance(transformer_template, type):
                 transformer = transformer_template()
@@ -339,20 +426,62 @@ class Table:
             LOGGER.debug('Loading transformer %s for field %s',
                          transformer.__class__.__name__, name)
             transformers[name] = transformer
+            sdtypes[name] = self._DTYPES_TO_TYPES.get(dtype, {}).get('type', 'categorical')
 
-        return transformers
+        return {'sdtypes': sdtypes, 'transformers': transformers}
+
+    def _fit_constraints(self, data):
+        errors = []
+        for constraint in self._constraints:
+            try:
+                constraint.fit(data)
+            except Exception as e:
+                errors.append(e)
+
+        if errors:
+            raise MultipleConstraintsErrors('\n' + '\n\n'.join(map(str, errors)))
+
+    def _transform_constraints(self, data, is_condition=False):
+        errors = []
+        if not is_condition:
+            self._constraints_to_reverse = []
+
+        for constraint in self._constraints:
+            try:
+                data = constraint.transform(data)
+                if not is_condition:
+                    self._constraints_to_reverse.append(constraint)
+
+            except (MissingConstraintColumnError, FunctionError) as e:
+                if isinstance(e, MissingConstraintColumnError):
+                    warnings.warn(
+                        f'{constraint.__class__.__name__} cannot be transformed because columns: '
+                        f'{e.missing_columns} were not found. Using the reject sampling approach '
+                        'instead.'
+                    )
+                else:
+                    warnings.warn(
+                        f'Error transforming {constraint.__class__.__name__}. '
+                        'Using the reject sampling approach instead.'
+                    )
+                if is_condition:
+                    indices_to_drop = data.columns.isin(constraint.constraint_columns)
+                    columns_to_drop = data.columns.where(indices_to_drop).dropna()
+                    data = data.drop(columns_to_drop, axis=1)
+
+            except Exception as e:
+                errors.append(e)
+
+        if errors:
+            raise MultipleConstraintsErrors('\n' + '\n\n'.join(map(str, errors)))
+
+        return data
 
     def _fit_transform_constraints(self, data):
-        for idx, constraint in enumerate(self._constraints):
-            if isinstance(constraint, type):
-                constraint = constraint().to_dict()
-            elif isinstance(constraint, Constraint):
-                constraint = constraint.to_dict()
-
-            constraint = Constraint.from_dict(constraint)
-            self._constraints[idx] = constraint
-
-            data = constraint.fit_transform(data)
+        # Fit and validate all constraints first because `transform` might change columns
+        # making the following constraints invalid
+        self._fit_constraints(data)
+        data = self._transform_constraints(data)
 
         return data
 
@@ -375,15 +504,31 @@ class Table:
         """
         meta_dtypes = self.get_dtypes(ids=False)
         dtypes = {}
+        numerical_extras = []
         for column in data.columns:
             if column in meta_dtypes:
                 dtypes[column] = meta_dtypes[column]
             elif column in extra_columns:
-                dtypes[column] = data[column].dtype.kind
+                dtype_kind = data[column].dtype.kind
+                if dtype_kind in ('i', 'f'):
+                    numerical_extras.append(column)
+                else:
+                    dtypes[column] = dtype_kind
 
-        transformers_dict = self._get_transformers(dtypes)
-        self._hyper_transformer = rdt.HyperTransformer(transformers=transformers_dict)
-        self._hyper_transformer.fit(data[list(dtypes.keys())])
+        ht_config = self._get_hypertransformer_config(dtypes)
+        for column in numerical_extras:
+            dtypes[column] = 'numerical'
+            ht_config['sdtypes'][column] = 'numerical'
+            ht_config['transformers'][column] = rdt.transformers.FloatFormatter(
+                missing_value_replacement='mean',
+                model_missing_values=True,
+            )
+
+        self._hyper_transformer = rdt.HyperTransformer()
+        self._hyper_transformer.set_config(ht_config)
+        fit_columns = list(dtypes)
+        if not data[fit_columns].empty:
+            self._hyper_transformer.fit(data[fit_columns])
 
     @staticmethod
     def _get_key_subtype(field_meta):
@@ -442,16 +587,13 @@ class Table:
         mappings = {}
         for name, field_metadata in self._fields_metadata.items():
             if field_metadata['type'] != 'id' and field_metadata.get('pii'):
-                faker = self._get_faker(field_metadata['pii_category'])
-
                 uniques = data[name].unique()
-                fake_values = [faker() for _ in range(len(uniques))]
-                mappings[name] = dict(zip(uniques, fake_values))
+                mappings[name] = dict(
+                    zip(uniques, Table._get_fake_values(field_metadata, len(uniques)))
+                )
 
         self._ANONYMIZATION_MAPPINGS[id(self)] = mappings
 
-        return mappings
-
     def _anonymize(self, data):
         anonymization_mappings = self._ANONYMIZATION_MAPPINGS.get(id(self))
         if anonymization_mappings:
@@ -495,34 +637,12 @@ class Table:
         self._fit_hyper_transformer(constrained, extra_columns)
         self.fitted = True
 
-    def _transform_constraints(self, data, on_missing_column='error'):
-        for constraint in self._constraints:
-            try:
-                data = constraint.transform(data)
-            except MissingConstraintColumnError:
-                if on_missing_column == 'error':
-                    raise MissingConstraintColumnError()
-
-                elif on_missing_column == 'drop':
-                    indices_to_drop = data.columns.isin(constraint.constraint_columns)
-                    columns_to_drop = data.columns.where(indices_to_drop).dropna()
-                    data = data.drop(columns_to_drop, axis=1)
-
-                else:
-                    raise ValueError('on_missing_column must be \'drop\' or \'error\'')
-
-        return data
-
-    def transform(self, data, on_missing_column='error'):
+    def transform(self, data, is_condition=False):
         """Transform the given data.
 
         Args:
             data (pandas.DataFrame):
                 Table data.
-            on_missing_column (str):
-                If the value is error, then a `MissingConstraintColumnError` is raised.
-                If the value is drop, then the columns involved in the constraint that
-                are present in data will be dropped.
 
         Returns:
             pandas.DataFrame:
@@ -536,10 +656,13 @@ class Table:
         data = self._anonymize(data[fields])
 
         LOGGER.debug('Transforming constraints for table %s', self.name)
-        data = self._transform_constraints(data, on_missing_column)
+        data = self._transform_constraints(data, is_condition)
 
         LOGGER.debug('Transforming table %s', self.name)
-        return self._hyper_transformer.transform(data)
+        try:
+            return self._hyper_transformer.transform_subset(data)
+        except (rdt.errors.NotFittedError, rdt.errors.Error):
+            return data
 
     @classmethod
     def _make_ids(cls, field_metadata, length):
@@ -571,9 +694,21 @@ class Table:
         if not self.fitted:
             raise MetadataNotFittedError()
 
-        reversed_data = self._hyper_transformer.reverse_transform(data)
+        reversible_columns = [
+            column
+            for column in self._hyper_transformer._output_columns
+            if column in data.columns
+        ]
+        reversed_data = data
+        try:
+            if not data.empty:
+                reversed_data = self._hyper_transformer.reverse_transform_subset(
+                    data[reversible_columns]
+                )
+        except rdt.errors.NotFittedError:
+            LOGGER.info('HyperTransformer has not been fitted for table %s', self.name)
 
-        for constraint in reversed(self._constraints):
+        for constraint in reversed(self._constraints_to_reverse):
             reversed_data = constraint.reverse_transform(reversed_data)
 
         for name, field_metadata in self._fields_metadata.items():
@@ -581,11 +716,13 @@ class Table:
             if field_type == 'id' and name not in reversed_data:
                 field_data = self._make_ids(field_metadata, len(reversed_data))
             elif field_metadata.get('pii', False):
-                faker = self._get_faker(field_metadata['pii_category'])
-                field_data = pd.Series([faker() for _ in range(len(reversed_data))])
+                field_data = pd.Series(Table._get_fake_values(field_metadata, len(reversed_data)))
             else:
                 field_data = reversed_data[name]
 
+            if field_metadata['type'] == 'numerical' and field_metadata['subtype'] == 'integer':
+                field_data = field_data.round()
+
             reversed_data[name] = field_data[field_data.notnull()].astype(self._dtypes[name])
 
         return reversed_data[self._field_names]
@@ -683,6 +820,8 @@ class Table:
             entity_columns=metadata_dict.get('entity_columns') or [],
             context_columns=metadata_dict.get('context_columns') or [],
             dtype_transformers=dtype_transformers,
+            enforce_min_max_values=metadata_dict.get('enforce_min_max_values', True),
+            learn_rounding_scheme=metadata_dict.get('learn_rounding_scheme', True),
         )
         instance._fields_metadata = fields
         return instance
