@@ -2,12 +2,14 @@
 
 """Main SDV module."""
 
-import pickle
 import warnings
 
+import cloudpickle
+
 from sdv.errors import NotFittedError
 from sdv.relational.hma import HMA1
 from sdv.tabular.copulas import GaussianCopula
+from sdv.utils import get_package_versions, throw_version_mismatch_warning
 
 
 class SDV:
@@ -20,7 +22,8 @@ class SDV:
             Class of the model to use. Defaults to ``sdv.relational.HMA1``.
         model_kwargs (dict):
             Keyword arguments to pass to the model. If no ``model`` is given,
-            this defaults to using a ``GaussianCopula`` with ``gaussian`` distribution.
+            this defaults to using a ``GaussianCopula`` with ``gaussian`` distribution
+            and ``FrequencyEncoder_noised`` categorical transformer.
     """
 
     _model_instance = None
@@ -28,7 +31,8 @@ class SDV:
     DEFAULT_MODEL_KWARGS = {
         'model': GaussianCopula,
         'model_kwargs': {
-            'distribution': 'gaussian'
+            'default_distribution': 'gaussian',
+            'categorical_transformer': 'FrequencyEncoder_noised',
         }
     }
 
@@ -139,14 +143,16 @@ class SDV:
         return self.sample(num_rows=num_rows, reset_primary_keys=reset_primary_keys)
 
     def save(self, path):
-        """Save this SDV instance to the given path using pickle.
+        """Save this SDV instance to the given path using cloudpickle.
 
         Args:
             path (str):
                 Path where the SDV instance will be serialized.
         """
+        self._package_versions = get_package_versions(getattr(self, '_model', None))
+
         with open(path, 'wb') as output:
-            pickle.dump(self, output)
+            cloudpickle.dump(self, output)
 
     @classmethod
     def load(cls, path):
@@ -157,4 +163,7 @@ class SDV:
                 Path from which to load the SDV instance.
         """
         with open(path, 'rb') as f:
-            return pickle.load(f)
+            model = cloudpickle.load(f)
+            throw_version_mismatch_warning(getattr(model, '_package_versions', None))
+
+            return model
