@@ -1,6 +1,7 @@
 import io
-from unittest.mock import MagicMock, Mock, patch
+from unittest.mock import MagicMock, Mock, mock_open, patch
 
+import cloudpickle
 import numpy as np
 import pandas as pd
 import pytest
@@ -59,29 +60,31 @@ class TestTabularPreset:
         gaussian_copula_mock.assert_called_once_with(
             table_metadata=None,
             constraints=None,
-            categorical_transformer='categorical_fuzzy',
+            categorical_transformer='FrequencyEncoder_noised',
             default_distribution='gaussian',
-            rounding=None,
+            learn_rounding_scheme=False,
         )
         metadata = gaussian_copula_mock.return_value._metadata
         metadata._dtype_transformers.update.assert_called_once_with({
-            'i': transformers_mock.NumericalTransformer(
-                dtype=np.int64,
-                nan=None,
-                null_column=False,
-                min_value='auto',
-                max_value='auto',
+            'i': transformers_mock.FloatFormatter(
+                missing_value_replacement=None,
+                model_missing_values=False,
+                enforce_min_max_values=True,
             ),
-            'f': transformers_mock.NumericalTransformer(
-                dtype=np.float64,
-                nan=None,
-                null_column=False,
-                min_value='auto',
-                max_value='auto',
+            'f': transformers_mock.FloatFormatter(
+                missing_value_replacement=None,
+                model_missing_values=False,
+                enforce_min_max_values=True,
             ),
-            'O': transformers_mock.CategoricalTransformer(fuzzy=True),
-            'b': transformers_mock.BooleanTransformer(nan=None, null_column=False),
-            'M': transformers_mock.DatetimeTransformer(nan=None, null_column=False),
+            'O': transformers_mock.FrequencyEncoder(add_noise=True),
+            'b': transformers_mock.BinaryEncoder(
+                missing_value_replacement=None,
+                model_missing_values=False
+            ),
+            'M': transformers_mock.UnixTimestampEncoder(
+                missing_value_replacement=None,
+                model_missing_values=False
+            )
         })
 
     @patch('sdv.lite.tabular.GaussianCopula', spec_set=GaussianCopula)
@@ -105,9 +108,9 @@ class TestTabularPreset:
         gaussian_copula_mock.assert_called_once_with(
             table_metadata=metadata.to_dict(),
             constraints=None,
-            categorical_transformer='categorical_fuzzy',
+            categorical_transformer='FrequencyEncoder_noised',
             default_distribution='gaussian',
-            rounding=None,
+            learn_rounding_scheme=False,
         )
 
     @patch('sdv.lite.tabular.rdt.transformers')
@@ -133,29 +136,31 @@ class TestTabularPreset:
         gaussian_copula_mock.assert_called_once_with(
             table_metadata=None,
             constraints=[constraint],
-            categorical_transformer='categorical_fuzzy',
+            categorical_transformer='FrequencyEncoder_noised',
             default_distribution='gaussian',
-            rounding=None,
+            learn_rounding_scheme=False,
         )
         metadata = gaussian_copula_mock.return_value._metadata
         metadata._dtype_transformers.update.assert_called_once_with({
-            'i': transformers_mock.NumericalTransformer(
-                dtype=np.int64,
-                nan='mean',
-                null_column=None,
-                min_value='auto',
-                max_value='auto',
+            'i': transformers_mock.FloatFormatter(
+                missing_value_replacement='mean',
+                model_missing_values=False,
+                enforce_min_max_values=True,
             ),
-            'f': transformers_mock.NumericalTransformer(
-                dtype=np.float64,
-                nan='mean',
-                null_column=None,
-                min_value='auto',
-                max_value='auto',
+            'f': transformers_mock.FloatFormatter(
+                missing_value_replacement='mean',
+                model_missing_values=False,
+                enforce_min_max_values=True,
             ),
-            'O': transformers_mock.CategoricalTransformer(fuzzy=True),
-            'b': transformers_mock.BooleanTransformer(nan=-1, null_column=None),
-            'M': transformers_mock.DatetimeTransformer(nan='mean', null_column=None),
+            'O': transformers_mock.FrequencyEncoder(add_noise=True),
+            'b': transformers_mock.BinaryEncoder(
+                missing_value_replacement=-1,
+                model_missing_values=False
+            ),
+            'M': transformers_mock.UnixTimestampEncoder(
+                missing_value_replacement='mean',
+                model_missing_values=False
+            )
         })
         assert preset._null_column is True
 
@@ -186,9 +191,9 @@ class TestTabularPreset:
         gaussian_copula_mock.assert_called_once_with(
             table_metadata=expected_metadata,
             constraints=None,
-            categorical_transformer='categorical_fuzzy',
+            categorical_transformer='FrequencyEncoder_noised',
             default_distribution='gaussian',
-            rounding=None,
+            learn_rounding_scheme=False,
         )
         metadata = gaussian_copula_mock.return_value._metadata
         assert metadata._dtype_transformers.update.call_count == 1
@@ -332,7 +337,7 @@ class TestTabularPreset:
         TabularPreset.sample(preset, 5)
 
         # Assert
-        model.sample.assert_called_once_with(5, True, None, None, None)
+        model.sample.assert_called_once_with(5, True, 100, None, None, None)
 
     def test_sample_conditions(self):
         """Test the ``TabularPreset.sample_conditions`` method.
@@ -358,6 +363,28 @@ class TestTabularPreset:
         # Assert
         model.sample_conditions.assert_called_once_with(conditions, 100, None, True, None)
 
+    def test_sample_conditions_with_max_tries(self):
+        """Test the ``TabularPreset.sample_conditions`` method with max tries.
+
+        Expect that the model's sample_conditions method is called with the expected args.
+
+        Input:
+        - num_rows=5
+        - max_retries=2
+        """
+        # Setup
+        model = MagicMock(spec=GaussianCopula)
+        preset = Mock()
+        preset._model = model
+        preset._null_percentages = None
+        conditions = [Mock()]
+
+        # Run
+        TabularPreset.sample_conditions(preset, conditions, max_tries_per_batch=2, batch_size=5)
+
+        # Assert
+        model.sample_conditions.assert_called_once_with(conditions, 2, 5, True, None)
+
     def test_sample_remaining_columns(self):
         """Test the ``TabularPreset.sample_remaining_columns`` method.
 
@@ -382,6 +409,29 @@ class TestTabularPreset:
         # Assert
         model.sample_remaining_columns.assert_called_once_with(conditions, 100, None, True, None)
 
+    def test_sample_remaining_columns_with_max_tries(self):
+        """Test the ``TabularPreset.sample_remaining_columns`` method with max tries.
+
+        Expect that the model's sample_remaining_columns method is called with the expected args.
+
+        Input:
+        - num_rows=5
+        - max_retries=2
+        """
+        # Setup
+        model = MagicMock(spec=GaussianCopula)
+        preset = Mock()
+        preset._model = model
+        preset._null_percentages = None
+        conditions = pd.DataFrame({'a': [1, 2, 3]})
+
+        # Run
+        TabularPreset.sample_remaining_columns(
+            preset, conditions, max_tries_per_batch=2, batch_size=5)
+
+        # Assert
+        model.sample_remaining_columns.assert_called_once_with(conditions, 2, 5, True, None)
+
     def test_list_available_presets(self):
         """Tests the ``TabularPreset.list_available_presets`` method.
 
@@ -403,7 +453,8 @@ class TestTabularPreset:
         # Assert
         assert out.getvalue().strip() == expected
 
-    def test_save(self):
+    @patch('sdv.lite.tabular.cloudpickle')
+    def test_save(self, cloudpickle_mock):
         """Test the ``TabularPreset.save`` method.
 
         Expect that the model's save method is called with the expected args.
@@ -418,14 +469,18 @@ class TestTabularPreset:
         model = Mock()
         preset = Mock()
         preset._model = model
+        open_mock = mock_open(read_data=cloudpickle.dumps('test'))
 
         # Run
-        TabularPreset.save(preset, 'test-path')
+        with patch('sdv.lite.tabular.open', open_mock):
+            TabularPreset.save(preset, 'test-path')
 
         # Assert
-        model.save.assert_called_once_with('test-path')
+        open_mock.assert_called_once_with('test-path', 'wb')
+        cloudpickle_mock.dump.assert_called_once_with(preset, open_mock())
 
-    def test_load(self):
+    @patch('sdv.lite.tabular.cloudpickle')
+    def test_load(self, cloudpickle_mock):
         """Test the ``TabularPreset.load`` method.
 
         Expect that the model's load method is called with the expected args.
@@ -439,12 +494,15 @@ class TestTabularPreset:
         # Setup
         default_model = Mock()
         TabularPreset._default_model = default_model
+        open_mock = mock_open(read_data=cloudpickle.dumps('test'))
 
         # Run
-        TabularPreset.load('test-file.pkl')
+        with patch('sdv.lite.tabular.open', open_mock):
+            loaded = TabularPreset.load('test-file.pkl')
 
         # Assert
-        default_model.load.called_once_with('test-file.pkl')
+        open_mock.assert_called_once_with('test-file.pkl', 'rb')
+        assert loaded == cloudpickle_mock.load.return_value
 
     def test___repr__(self):
         """Test the ``TabularPreset.__repr__`` method.
